@@ -289,6 +289,7 @@ void tst_QWebSocketServer::tst_listening()
 #ifndef QT_NO_SSL
     QSignalSpy peerVerifyErrorSpy(&server, SIGNAL(peerVerifyError(QSslError)));
     QSignalSpy sslErrorsSpy(&server, SIGNAL(sslErrors(QList<QSslError>)));
+    QSignalSpy sslErrorsOccurredSpy(&server, SIGNAL(sslErrorsOccurred(QSslSocket*, QList<QSslError>)));
 #endif
 
     QVERIFY(server.listen());   //listen on all network interface, choose an appropriate port
@@ -310,6 +311,7 @@ void tst_QWebSocketServer::tst_listening()
 #ifndef QT_NO_SSL
     QCOMPARE(peerVerifyErrorSpy.size(), 0);
     QCOMPARE(sslErrorsSpy.size(), 0);
+    QCOMPARE(sslErrorsOccurredSpy.size(), 0);
 #endif
     QCOMPARE(serverErrorSpy.size(), 1);
     QCOMPARE(serverClosedSpy.size(), 1);
@@ -330,6 +332,7 @@ void tst_QWebSocketServer::tst_connectivity()
 #ifndef QT_NO_SSL
     QSignalSpy peerVerifyErrorSpy(&server, SIGNAL(peerVerifyError(QSslError)));
     QSignalSpy sslErrorsSpy(&server, SIGNAL(sslErrors(QList<QSslError>)));
+    QSignalSpy sslErrorsOccurredSpy(&server, SIGNAL(sslErrorsOccurred(QSslSocket*, QList<QSslError>)));
 #endif
     QWebSocket socket;
     QSignalSpy socketConnectedSpy(&socket, SIGNAL(connected()));
@@ -339,7 +342,7 @@ void tst_QWebSocketServer::tst_connectivity()
     QCOMPARE(server.serverUrl(), QUrl(QStringLiteral("ws://") + QHostAddress(QHostAddress::LocalHost).toString() +
                                  QStringLiteral(":").append(QString::number(server.serverPort()))));
 
-    socket.open(server.serverUrl());
+    socket.open(server.serverUrl().toString());
 
     QTRY_COMPARE(socketConnectedSpy.size(), 1);
     QCOMPARE(socket.state(), QAbstractSocket::ConnectedState);
@@ -354,6 +357,7 @@ void tst_QWebSocketServer::tst_connectivity()
 #ifndef QT_NO_SSL
     QCOMPARE(peerVerifyErrorSpy.size(), 0);
     QCOMPARE(sslErrorsSpy.size(), 0);
+    QCOMPARE(sslErrorsOccurredSpy.size(), 0);
 #endif
     QCOMPARE(serverErrorSpy.size(), 0);
 }
@@ -460,6 +464,7 @@ void tst_QWebSocketServer::tst_preSharedKey()
                               SIGNAL(serverError(QWebSocketProtocol::CloseCode)));
     QSignalSpy serverClosedSpy(&server, &QWebSocketServer::closed);
     QSignalSpy sslErrorsSpy(&server, SIGNAL(sslErrors(QList<QSslError>)));
+    QSignalSpy sslErrorsOccurredSpy(&server, SIGNAL(sslErrorsOccurred(QSslSocket*, QList<QSslError>)));
 
     QWebSocket socket;
     QSslConfiguration socketConfig = QSslConfiguration::defaultConfiguration();
@@ -481,7 +486,7 @@ void tst_QWebSocketServer::tst_preSharedKey()
     QCOMPARE(server.serverUrl(), QUrl(QString::asprintf("wss://%ls:%d",
                                  qUtf16Printable(QHostAddress(QHostAddress::LocalHost).toString()), server.serverPort())));
 
-    socket.open(server.serverUrl());
+    socket.open(server.serverUrl().toString());
 
     QTRY_COMPARE(socketConnectedSpy.size(), 1);
     QCOMPARE(socket.state(), QAbstractSocket::ConnectedState);
@@ -495,6 +500,7 @@ void tst_QWebSocketServer::tst_preSharedKey()
 
     QTRY_COMPARE(serverClosedSpy.size(), 1);
     QCOMPARE(sslErrorsSpy.size(), 0);
+    QCOMPARE(sslErrorsOccurredSpy.size(), 0);
     QCOMPARE(serverErrorSpy.size(), 0);
 }
 #endif
@@ -517,6 +523,7 @@ void tst_QWebSocketServer::tst_maxPendingConnections()
 #ifndef QT_NO_SSL
     QSignalSpy peerVerifyErrorSpy(&server, SIGNAL(peerVerifyError(QSslError)));
     QSignalSpy sslErrorsSpy(&server, SIGNAL(sslErrors(QList<QSslError>)));
+    QSignalSpy sslErrorsOccurredSpy(&server, SIGNAL(sslErrorsOccurred(QSslSocket*, QList<QSslError>)));
 #endif
     QSignalSpy serverAcceptErrorSpy(&server, SIGNAL(acceptError(QAbstractSocket::SocketError)));
 
@@ -530,18 +537,18 @@ void tst_QWebSocketServer::tst_maxPendingConnections()
 
     QVERIFY(server.listen());
 
-    socket1.open(server.serverUrl());
+    socket1.open(server.serverUrl().toString());
 
     QTRY_COMPARE(socket1ConnectedSpy.size(), 1);
     QCOMPARE(socket1.state(), QAbstractSocket::ConnectedState);
     QCOMPARE(serverConnectionSpy.size(), 1);
     QCOMPARE(corsAuthenticationSpy.size(), 1);
-    socket2.open(server.serverUrl());
+    socket2.open(server.serverUrl().toString());
     QTRY_COMPARE(socket2ConnectedSpy.size(), 1);
     QCOMPARE(socket2.state(), QAbstractSocket::ConnectedState);
     QCOMPARE(serverConnectionSpy.size(), 2);
     QCOMPARE(corsAuthenticationSpy.size(), 2);
-    socket3.open(server.serverUrl());
+    socket3.open(server.serverUrl().toString());
     QVERIFY(!socket3ConnectedSpy.wait(250));
     QCOMPARE(socket3ConnectedSpy.size(), 0);
     QCOMPARE(socket3.state(), QAbstractSocket::UnconnectedState);
@@ -573,6 +580,7 @@ void tst_QWebSocketServer::tst_maxPendingConnections()
 #ifndef QT_NO_SSL
     QCOMPARE(peerVerifyErrorSpy.size(), 0);
     QCOMPARE(sslErrorsSpy.size(), 0);
+    QCOMPARE(sslErrorsOccurredSpy.size(), 0);
 #endif
     QCOMPARE(serverAcceptErrorSpy.size(), 0);
 }
@@ -597,7 +605,7 @@ void tst_QWebSocketServer::tst_serverDestroyedWhileSocketConnected()
     QCOMPARE(server->serverUrl(), QUrl(QStringLiteral("ws://") + QHostAddress(QHostAddress::LocalHost).toString() +
                                   QStringLiteral(":").append(QString::number(server->serverPort()))));
 
-    socket.open(server->serverUrl());
+    socket.open(server->serverUrl().toString());
 
     QTRY_COMPARE(socketConnectedSpy.size(), 1);
     QCOMPARE(socket.state(), QAbstractSocket::ConnectedState);
@@ -641,7 +649,7 @@ void tst_QWebSocketServer::tst_scheme()
     QVERIFY(plainServer.listen());
 
     QWebSocket plainSocket;
-    plainSocket.open(plainServer.serverUrl());
+    plainSocket.open(plainServer.serverUrl().toString());
 
     QTRY_COMPARE(plainServerConnectionSpy.size(), 1);
     QScopedPointer<QWebSocket> plainServerSocket(plainServer.nextPendingConnection());
@@ -665,7 +673,7 @@ void tst_QWebSocketServer::tst_scheme()
                 secureSocket.ignoreSslErrors();
                 sessionCipher = secureSocket.sslConfiguration().sessionCipher();
             });
-    secureSocket.open(secureServer.serverUrl());
+    secureSocket.open(secureServer.serverUrl().toString());
 
     QTRY_COMPARE(secureServerConnectionSpy.size(), 1);
     QScopedPointer<QWebSocket> secureServerSocket(secureServer.nextPendingConnection());
@@ -673,6 +681,35 @@ void tst_QWebSocketServer::tst_scheme()
     QCOMPARE(secureServerSocket->requestUrl().scheme(), QStringLiteral("wss"));
     secureServer.close();
     QVERIFY(!sessionCipher.isNull());
+
+    QWebSocketServer selfSignedServer(QString(), QWebSocketServer::SecureMode);
+    QSslConfiguration sslConfiguration;
+    QFile certFile(QStringLiteral(":/selfsigned.cert"));
+    QFile keyFile(QStringLiteral(":/selfsigned.key"));
+    QVERIFY(certFile.open(QIODevice::ReadOnly));
+    QVERIFY(keyFile.open(QIODevice::ReadOnly));
+    QSslCertificate certificate(&certFile, QSsl::Pem);
+    QSslKey sslKey(&keyFile, QSsl::Rsa, QSsl::Pem);
+    certFile.close();
+    keyFile.close();
+    sslConfiguration.setLocalCertificate(certificate);
+    sslConfiguration.setPrivateKey(sslKey);
+    sslConfiguration.setPeerVerifyMode(QSslSocket::VerifyPeer);
+    selfSignedServer.setSslConfiguration(sslConfiguration);
+    QSignalSpy selfSignedServerConnectionSpy(&selfSignedServer, SIGNAL(newConnection()));
+    QSignalSpy selfSignedServerSslErrorsOccurredSpy(
+        &selfSignedServer, SIGNAL(sslErrorsOccurred(QSslSocket*, QList<QSslError>)));
+
+    QVERIFY(selfSignedServer.listen());
+
+    QWebSocket selfSignedSocket;
+    sslConfiguration.setPeerVerifyMode(QSslSocket::VerifyNone);
+    selfSignedSocket.setSslConfiguration(sslConfiguration);
+    QObject::connect(&selfSignedSocket, &QWebSocket::sslErrors,
+                     &selfSignedSocket, qOverload<>(&QWebSocket::ignoreSslErrors));
+    selfSignedSocket.open(selfSignedServer.serverUrl().toString());
+
+    QTRY_COMPARE(selfSignedServerSslErrorsOccurredSpy.size(), 1);
 #endif
 }
 
@@ -753,7 +790,7 @@ void tst_QWebSocketServer::tst_handshakeTimeout()
         QVERIFY(plainServer.listen());
 
         QWebSocket socket;
-        socket.open(plainServer.serverUrl());
+        socket.open(plainServer.serverUrl().toString());
 
         QTRY_COMPARE(plainServerConnectionSpy.size(), 1);
         QScopedPointer<QWebSocket> plainServerSocket(plainServer.nextPendingConnection());
@@ -783,7 +820,7 @@ void tst_QWebSocketServer::tst_handshakeTimeout()
         QCOMPARE(plainServerConnectionSpy.size(), 0);
 
         QWebSocket socket;
-        socket.open(plainServer.serverUrl());
+        socket.open(plainServer.serverUrl().toString());
 
         /* Check that a real websocket will be processed after some non-websocket
          * TCP connections timeout. */
@@ -827,7 +864,7 @@ void tst_QWebSocketServer::tst_handshakeTimeout()
         config.setPeerVerifyMode(QSslSocket::VerifyNone);
         secureSocket.setSslConfiguration(config);
 
-        secureSocket.open(secureServer.serverUrl());
+        secureSocket.open(secureServer.serverUrl().toString());
 
         QTRY_COMPARE(secureServerConnectionSpy.size(), 1);
         QScopedPointer<QWebSocket> serverSocket(secureServer.nextPendingConnection());
@@ -849,7 +886,7 @@ void tst_QWebSocketServer::tst_handshakeTimeout()
         QWebSocket socket;
         QSignalSpy socketConnectedSpy(&socket, &QWebSocket::connected);
         QSignalSpy socketDisconnectedSpy(&socket, &QWebSocket::disconnected);
-        socket.open(plainServer.serverUrl());
+        socket.open(plainServer.serverUrl().toString());
 
         QTRY_COMPARE(plainServerConnectionSpy.size(), 1);
         QTRY_COMPARE(socketConnectedSpy.size(), 1);
@@ -902,7 +939,7 @@ void tst_QWebSocketServer::multipleFrames()
     QWebSocket socket;
     QSignalSpy socketConnectedSpy(&socket, &QWebSocket::connected);
     QSignalSpy messageReceivedSpy(&socket, &QWebSocket::binaryMessageReceived);
-    socket.open(server.serverUrl());
+    socket.open(server.serverUrl().toString());
 
     QVERIFY(serverConnectionSpy.wait());
     QVERIFY(socketConnectedSpy.wait());
